@@ -2,7 +2,7 @@
 
 -- |
 -- Module:      Data.Aeson.Parser.Internal
--- Copyright:   (c) 2011 Bryan O'Sullivan
+-- Copyright:   (c) 2011, 2012 Bryan O'Sullivan
 --              (c) 2011 MailRank, Inc.
 -- License:     Apache
 -- Maintainer:  Bryan O'Sullivan <bos@serpentine.com>
@@ -23,6 +23,7 @@ module Data.Aeson.Parser.Internal
     , value'
     -- * Helpers
     , decodeWith
+    , eitherDecodeWith
     ) where
 
 import Blaze.ByteString.Builder (fromByteString, toByteString)
@@ -238,6 +239,15 @@ decodeWith p to s =
       _          -> Nothing
 {-# INLINE decodeWith #-}
 
+eitherDecodeWith :: Parser Value -> (Value -> Result a) -> L.ByteString -> Either String a
+eitherDecodeWith p to s =
+    case L.parse p s of
+      L.Done _ v -> case to v of
+                      Success a -> Right a
+                      Error msg -> Left msg
+      L.Fail _ _ msg -> Left msg
+{-# INLINE eitherDecodeWith #-}
+
 -- $lazy
 --
 -- The 'json' and 'value' parsers decouple identification from
