@@ -2,7 +2,7 @@
 
 -- |
 -- Module:      Data.Aeson.Types
--- Copyright:   (c) 2011 Bryan O'Sullivan
+-- Copyright:   (c) 2011, 2012 Bryan O'Sullivan
 --              (c) 2011 MailRank, Inc.
 -- License:     Apache
 -- Maintainer:  Bryan O'Sullivan <bos@serpentine.com>
@@ -32,6 +32,14 @@ module Data.Aeson.Types
     , parseEither
     , parseMaybe
     , ToJSON(..)
+
+    -- * Inspecting @'Value's@
+    , withObject
+    , withText
+    , withArray
+    , withNumber
+    , withBool
+
     -- * Constructors and accessors
     , (.=)
     , (.:)
